﻿using Correo.Abstractions;
using Correo.Application.Utils;
using System;
using System.Linq;

namespace Correo.Application.Extensions
{
    internal static class ValidationExtensions
    {
        public static void Validate(this EmailMessage emailMessage)
        {
            if (string.IsNullOrEmpty(emailMessage.Subject))
                throw new Exception("Subject is missing.");

            if (string.IsNullOrEmpty(emailMessage.Body))
                throw new Exception("Body is missing.");

            if (string.IsNullOrEmpty(emailMessage.From.Address))
                throw new Exception("'From' address is missing.");

            if (!Validations.IsEmail(emailMessage.From.Address))
                throw new Exception($"'From' email address '{emailMessage.From.Address}' is not valid.");

            if (emailMessage.To.Count() == 0)
                throw new Exception("'To' addresses are empty.");

            if (emailMessage.To.Any(z => !Validations.IsEmail(z.Address)))
                throw new Exception("There are invalid email addresses in 'To' list.");

            if (emailMessage.Cc.Any(z => !Validations.IsEmail(z.Address)))
                throw new Exception("There are invalid email addresses in 'Cc' list.");

            if (emailMessage.Bcc.Any(z => !Validations.IsEmail(z.Address)))
                throw new Exception("There are invalid email addresses in 'Bcc' list.");
        }
    }
}
